/*
	(C) 1995-96 AROS - The Amiga Research OS
	$Id: endrefresh.c,v 1.3 2009/04/26 19:42:36 jacadcaps Exp $
 
	Desc:
	Lang: english
*/
#include <proto/layers.h>
#include <proto/graphics.h>
#include "intuition_intern.h"
#include "renderwindowframe.h"
#include "mosmisc.h"

/*****************************************************************************
 
	NAME */
#include <proto/intuition.h>

AROS_LH2(void, EndRefresh,

		 /*  SYNOPSIS */
		 AROS_LHA(struct Window *, window, A0),
		 AROS_LHA(BOOL           , complete, D0),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 61, Intuition)

/*  FUNCTION
 
	INPUTS
 
	RESULT
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
 
	INTERNALS
 
	HISTORY
	29-10-95    digulla automatically created from
			    intuition_lib.fd and clib/intuition_protos.h
 
*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)

	DEBUG_REFRESH(dprintf("EndRefresh: Window 0x%lx Complete %d\n", window, complete));

	SANITY_CHECK(window)

	/* Check whether the BeginRefresh was aborted due to a FALSE=BeginUpdate()*/
	if (window->Flags & WFLG_WINDOWREFRESH)
	{
		#ifndef DAMAGECACHE
		#ifdef BEGINUPDATEGADGETREFRESH
		if ((complete) && (window->MoreFlags & WMFLG_IAMMUI))
		{
			ULONG mode = NO_DOUBLEBUFFER;

			if (IW(window)->specialflags & SPFLAG_LAYERRESIZED) mode = DOUBLEBUFFER;
			IW(window)->specialflags &= ~(SPFLAG_LAYERREFRESH|SPFLAG_LAYERRESIZED);

			if (IW(window)->specialflags & SPFLAG_WANTBUFFER) mode = DOUBLEBUFFER;

			if (!IS_GZZWINDOW(window))
			{
				if (window->Flags & WFLG_BORDERLESS)
				{
					int_refreshglist(window->FirstGadget, window, NULL, -1, 0, 0, IntuitionBase);
				} else {
					if (mode != DOUBLEBUFFER)
					{
						/* we can check if it's possible to avoid border refresh here */
						if (iswindowborderdamaged(WLAYER(window),window,IntuitionBase))
						{
							int_RefreshWindowFrame(window,0,0,mode,IntuitionBase);
						} else {
							int_refreshglist(window->FirstGadget, window, NULL, -1, 0, REFRESHGAD_BORDER, IntuitionBase);
						}
					} else {
						int_RefreshWindowFrame(window,0,0,mode,IntuitionBase);
					}
				}
			} else {
				/* refresh all gadgets except border and gadtools gadgets */
				int_refreshglist(window->FirstGadget, window, NULL, -1, 0, REFRESHGAD_BORDER , IntuitionBase);
			}
		}
		#endif
		#endif

		EndUpdate(WLAYER(window), complete);
	}
	/* reset all bits indicating a necessary or ongoing refresh */
	window->Flags &= ~WFLG_WINDOWREFRESH;

	/* I reset this one only if Complete is TRUE!?! */
	if (TRUE == complete)
		WLAYER(window)->Flags &= ~LAYERREFRESH;

	/* Unlock the layers. */
	UnlockLayer(WLAYER(window));

	if (BLAYER(window))
		UnlockLayer(BLAYER(window));

	UNLOCKGADGET

	UnlockLayerInfo(&window->WScreen->LayerInfo);

	AROS_LIBFUNC_EXIT
} /* EndRefresh */
